"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.packFileBoxToChunk = exports.unpackFileBoxFromChunk = void 0;
const grpc_1 = require("@chatie/grpc");
const wechaty_puppet_1 = require("wechaty-puppet");
const stream_1 = require("stream");
const stronger_typed_streams_1 = require("stronger-typed-streams");
const next_data_1 = require("./next-data");
const decoder = () => new stronger_typed_streams_1.Transform({
    objectMode: true,
    transform: (chunk, _, callback) => {
        if (!chunk.hasData()) {
            callback(new Error('no data'));
            return;
        }
        const data = chunk.getData();
        callback(null, data);
    },
});
async function unpackFileBoxFromChunk(stream) {
    const chunk = await next_data_1.nextData(stream);
    if (!chunk.hasName()) {
        throw new Error('no name');
    }
    const fileName = chunk.getName();
    const fileStream = new stream_1.PassThrough({ objectMode: true });
    const transformedStream = stream.pipe(decoder());
    transformedStream.pipe(fileStream);
    stream.on('error', e => fileStream.emit('error', e));
    transformedStream.on('error', e => fileStream.emit('error', e));
    const fileBox = wechaty_puppet_1.FileBox.fromStream(fileStream, fileName);
    return fileBox;
}
exports.unpackFileBoxFromChunk = unpackFileBoxFromChunk;
const encoder = () => new stronger_typed_streams_1.Transform({
    objectMode: true,
    transform: (chunk, _, callback) => {
        const fileBoxChunk = new grpc_1.FileBoxChunk();
        fileBoxChunk.setData(chunk);
        callback(null, fileBoxChunk);
    },
});
async function packFileBoxToChunk(fileBox) {
    const stream = new stream_1.PassThrough({ objectMode: true });
    const chunk = new grpc_1.FileBoxChunk();
    chunk.setName(fileBox.name);
    // FIXME: Huan(202010) write might return false
    stream.write(chunk);
    fileBox
        .pipe(encoder())
        .pipe(stream);
    return stream;
}
exports.packFileBoxToChunk = packFileBoxToChunk;
//# sourceMappingURL=file-box-chunk.js.map