#!/usr/bin/env ts-node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tstest_1 = require("tstest");
const wechaty_puppet_mock_1 = require("wechaty-puppet-mock");
const src_1 = require("../src/");
tstest_1.test('integration testing', async (t) => {
    const TOKEN = 'test_token';
    const ENDPOINT = '0.0.0.0:8788';
    const DING = 'ding_data';
    /**
     * Puppet in Hostie
     */
    const puppet = new wechaty_puppet_mock_1.PuppetMock();
    const spyStart = tstest_1.sinon.spy(puppet, 'start');
    const spyOn = tstest_1.sinon.spy(puppet, 'on');
    const spyDing = tstest_1.sinon.spy(puppet, 'ding');
    /**
     * Hostie Server
     */
    const serverOptions = {
        endpoint: ENDPOINT,
        puppet: puppet,
        token: TOKEN,
    };
    const hostieServer = new src_1.PuppetServer(serverOptions);
    await hostieServer.start();
    /**
     * Puppet Hostie Client
     */
    const puppetOptions = {
        endpoint: ENDPOINT,
        token: TOKEN,
    };
    const puppetHostie = new src_1.PuppetHostie(puppetOptions);
    await puppetHostie.start();
    t.ok(spyStart.called, 'should called the hostie server start() function');
    const future = new Promise((resolve, reject) => {
        const offError = () => puppetHostie.off('error', reject);
        puppetHostie.once('dong', payload => {
            resolve(payload.data);
            offError();
        });
        puppetHostie.once('error', e => {
            reject(e);
            offError();
        });
    });
    puppetHostie.ding(DING);
    const result = await future;
    t.ok(spyOn.called, 'should called the hostie server on() function');
    t.ok(spyDing.called, 'should called the hostie server ding() function');
    t.equal(result, DING, 'should get a successful roundtrip for ding');
    /**
     * Stop
     *  1. Puppet in Hostie
     *  2. Hostie Service
     *  3. Puppet Hostie Client
     *
     */
    await puppetHostie.stop();
    await hostieServer.stop();
});
//# sourceMappingURL=integration.spec.js.map